/* This key only works from the Pro6PP domains, for demo purposes.
   Replace this auth_key with your own.
 */
var pro6pp_auth_key = "n5SEkGlaFzbkWDfj";

var icons = {
    "self" : {
        "url" : "pin_red.png",
        "width" : 16,
        "height" : 28
    },
    "small_store" : {
        "url" : "marker_green.png",
        "width" : 43,
        "height" : 37
    },
    "big_store" : {
        "url" : "marker_blue.png",
        "width" : 43,
        "height" : 37
    }
}

var stores_json = {
    "5652" : {
        "icon" : "big_store",
        "link" : "http://multicopy.nl/Eindhoven/",
        "title" : "MultiCopy Eindhoven",
        "phone" : "040-2377820",
        "address" : "Hurksestraat 28 B | 5652 AL, Eindhoven"
    },
    "1000" : {
        "icon" : "small_store",
        "link" : "http://multicopy.nl/Eindhoven/",
        "title" : "MultiCopy Amsterdam",
        "phone" : "040-2377820",
        "address" : "Hurksestraat 28 B | 5652 AL, Amsterdam"
    }
};

var showPopupOnHover = true;
var showPopupOnLoad = false;

var map = null;
var layer = null;
var start_lat = 52.3
var start_lng = 5.8
var zoom = 8;

$(document).ready(function() {
    // Lookup start fourpp and location based on city or sixpp
    $('.pro6pp_suggest').applySuggest({
        callback : function suggestCallback(result) {
            if (result.city) {
                storeLocate(result.nl_fourpps, result.lat, result.lng);
            }
        }
    });

    var postcode_error_label =  $('label.error');
    $('.pro6pp_autocomplete').applyAutocomplete({
        callback : function autocompleteCallback(json) {
            postcode_error_label.hide();
            if (json.status === 'ok') {
                for ( var i in json.results) {
                    var result = json.results[i];
                    storeLocate(result.nl_sixpp, result.lat, result.lng);
                }
            } else {
                postcode_error_label.show();
            }
        }
    });

    // Prepare list of stores for the locator call.
    var fourpps = "";
    for ( var postcode in stores_json) {
        fourpps += "," + postcode;
    }
    // Remove trailing comma
    fourpps = fourpps.substring(1);
    $('.target_nl_fourpps').val(fourpps);

    // When start fourpp is determined, look for nearby stores
    $('.pro6pp_locator_submit').click(function() {
        $('.pro6pp_suggest').locator({
            callback : locatorCallback
        });
        return false;
    });

    // Also load the map.
    addMap();
});

function storeLocate(fourpps, lat, lng) {
    var fourpp = fourpps.substring(0, 4);
    $('.nl_fourpp').val(fourpp);
    $('#startLocate').removeAttr("disabled");

    // Put our own location on map
    addMarker(layer, lng, lat, null, "self");

    var position = transform(lat, lng);
    map.setCenter(position, zoom);
}

function locatorCallback(results) {
    // Clear optional old results
    $('.postcodeResult').empty();

    for ( var i in results) {
        var result = results[i];

        var nl_fourpp = result.nl_fourpp;
        var store_info = stores_json[nl_fourpp];
        var distance_km = Math.round(result.distance / 1000);
        var template = getStoreTemplate(store_info, distance_km);
        $('.postcodeResult').append(template);

        // Put marker on map
        addMarker(layer, result.lng, result.lat, template, store_info.icon);
    }
}

function getStoreTemplate(store_info, distance_km) {
    var template = '<li> \
		<div style="border-radius: 5px 5px 5px 5px;" \
			class="corner lghtGray"> \
			<div class="lghtGraySpacer"> \
				<div class="away">'
            + distance_km
            + ' km vanaf uw locatie</div> \
				<a class="vestigingNaam" href="'
            + store_info.link
            + '">'
            + store_info.title
            + '</a><br> \
				<div class="vestigingTel">'
            + store_info.phone
            + '</div> \
				<div class="adresgegevens"> \
				'
            + store_info.address
            + ' \
				</div> \
			</div> \
		</div> \
	</li>';
    return template;
}

function transform(lat, lng) {
    // Transform from WGS 1984
    var fromProjection = new OpenLayers.Projection("EPSG:4326");
    // to Spherical Mercator Projection
    var toProjection = new OpenLayers.Projection("EPSG:900913");
    var position = new OpenLayers.LonLat(lng, lat)
    position = position.transform(fromProjection, toProjection);
    return position;
}

function addMap() {
    map = new OpenLayers.Map("map");
    var mapnik = new OpenLayers.Layer.OSM();

    var position = transform(start_lat, start_lng);

    map.addLayer(mapnik);
    map.setCenter(position, zoom);
    layer = new OpenLayers.Layer.Markers("POIs");
    map.addLayer(layer);
}

function Lon2Merc(lon) {
    return 20037508.34 * lon / 180;
}

function Lat2Merc(lat) {
    var PI = 3.14159265358979323846;
    lat = Math.log(Math.tan((90 + lat) * PI / 360)) / (PI / 180);
    return 20037508.34 * lat / 180;
}

function addMarker(layer, lon, lat, popupContentHTML, iconId) {
    var ll = new OpenLayers.LonLat(Lon2Merc(lon), Lat2Merc(lat));

    // Create feature (Popup and Marker)
    var feature = new OpenLayers.Feature(layer, ll);
    if (popupContentHTML !== null) {
        feature.closeBox = true;
        feature.popupClass = OpenLayers.Class(OpenLayers.Popup.FramedCloud, {
            minSize : new OpenLayers.Size(200, 120)
        });
        feature.data.popupContentHTML = popupContentHTML;
    }
    feature.data.overflow = "auto";
    feature.data.icon = makeIcon(iconId);

    var marker = feature.createMarker();

    if (popupContentHTML !== null) {
        // Click
        var markerClick = function(evt) {
            if (!this.popup.visible())
                this.popup.clicked = false;
            if (this.popup.clicked == true) {
                this.popup.clicked = false;
                this.popup.hide();
            } else {
                this.popup.clicked = true;
                if (!this.popup.visible())
                    this.popup.show();
            }
            OpenLayers.Event.stop(evt);
        };
        // Hover
        var markerHover = function(evt) {
            if (!this.popup.visible())
                this.popup.clicked = false;
            if (!this.popup.clicked)
                this.popup.show();

            OpenLayers.Event.stop(evt);
        }
        // Hover End
        var markerHoverEnd = function(evt) {
            if (!this.popup.clicked) {
                this.popup.hide();
            }
            OpenLayers.Event.stop(evt);
        }

        marker.events.register("mousedown", feature, markerClick);
        if (showPopupOnHover) {
            marker.events.register("mouseover", feature, markerHover);
            marker.events.register("mouseout", feature, markerHoverEnd);
        }
    }

    layer.addMarker(marker);
    map.addPopup(feature.createPopup(feature.closeBox));

    if (showPopupOnLoad != true) {
        feature.popup.hide();
        feature.popup.clicked = false;
    } else {
        feature.popup.clicked = true;
    }

    return marker;
}

function makeIcon(icon_id) {
    var i = icons[icon_id];

    icon_size = new OpenLayers.Size(i.width, i.height);
    icon_offset = new OpenLayers.Pixel(-icon_size.w / 2, -icon_size.h / 2);
    icon = new OpenLayers.Icon(i.url, icon_size, icon_offset);
    return icon
}
