// Create closure to keep namespace clean and hide implementation.
(function($) {
	$.fn.applySuggest = function(options) { 
        var instance = this;
        
        function getConfig(field) {
            if (typeof options === 'undefined' ||
            	typeof options[field] === 'undefined') {
                // Use default field class name
                return null;    
            } else {
                // Developer chose to specify form field manually.
                return options[field];    
            }
        } 
        instance.callback = getConfig('callback');
       
		suggest(instance);
		// Initialize selected city (default)
		this.city_key = null;
	};

	var pro6pp_cache = {};
	function pro6pp_cached_get(obj, url, callback, callback_parameters) {
		if (pro6pp_cache.hasOwnProperty(url)) {
			callback(obj, pro6pp_cache[url], callback_parameters || []);
		} else {
			$.getJSON(url + "&callback=?", function(json) {
				pro6pp_cache[url] = json;
				callback(obj, json, callback_parameters || []);
			});
		}
	}

	function suggest(obj) {
		var city = $(obj).find('.city');
		city.autocomplete({
			// Define callback to format results
			source : function(req, add) {
				var url = "http://api.pro6pp.nl/v1/suggest?auth_key="
						+ pro6pp_auth_key + "&per_page=5&nl_city=" + req.term;
				url = encodeURI(url);
				var parameters = [ add ];
				pro6pp_cached_get(obj, url, fillin, parameters);
			},
			select: function( event, ui ) {
				// When a city has been selected, remember the city_key, in order to ask for streets
				// in this specific city later on.
			    var result = ui.item.key;
				obj.city_key = result.city_key;
				fillin_one(obj, result);
				if (typeof obj.callback !== 'undefined') { 
		            // When pressing tab, make a selection.
		        	obj.callback(result);
		        }
				return true;
			}
		});

		var street = $(obj).find('.street');
		street.autocomplete({
			// Define callback to format results
			source : function(req, add) {
				var city = $(obj).find('.city').val();
				var url = "http://api.pro6pp.nl/v1/suggest?auth_key="
						+ pro6pp_auth_key + "&per_page=5&city_key=" + obj.city_key
						+ "&street=" + req.term;
				url = encodeURI(url);
				var parameters = [ add ];
				pro6pp_cached_get(obj, url, fillin, parameters);
			},
            select: function( event, ui ) {
                var result = ui.item.key;
                fillin_one(obj, result);
                return true;
            }
		});
	}

    function fillin_one(obj, result) {
        if (result.street) { 
            $(obj).find('.street').val(result.street);
            $(obj).find('.str_lat').val(result.lat);
            $(obj).find('.str_lng').val(result.lng);
            $(obj).find('.nl_sixpps').val(result.nl_sixpps);                    
        } else if (result.city) {
            $(obj).find('.official_city').val(result.official_city);
            $(obj).find('.province').val(result.province);
            $(obj).find('.lat').val(result.lat);
            $(obj).find('.lng').val(result.lng);
            $(obj).find('.nl_fourpps').val(result.nl_fourpps);
        } 
    }
    
	function fillin(obj, json, callback_parameters) {
		// The add function was the first parameter
		add = callback_parameters[0];
		var suggestions = [];

		// Clear possibly earlier displayed error messages
		$(obj).find('.city_message').empty();
		$(obj).find('.street_message').empty();

		if (json.status == 'ok') {
			$.each(json.results, function(i, result) {
			    if (result.street) {
		            suggestions.push({
                        label: result.street,
                        value: result.street,
                        key: result});
			    } else if (result.city) {
		            // Show nice city name to user at first using 'label'..
		            // e.g. Bergen (gem. Bergen, Limburg).
		            // After selecting the city, fill in the official name.
		            // Also keep the associated key to find streets later on.
			        suggestions.push({
			            label: result.city,
                        value: result.official_city,
                        key: result});
			    }
				fillin_one(obj, result);
			});
    
        	// Pass array to callback that builds the suggestion list
			add(suggestions);
		} else {
			var translated_message = json.error.message;
			if (json.error.message == 'city not found') {
        			// Invalidate any previously selected city
	        		obj.city_key = null;

				translated_message = 'Onbekende plaats';
				$(obj).find('.city_message').html(translated_message);
			} else if (json.error.message == 'street not found') {
				translated_message = 'Onbekende straat';
				$(obj).find('.street_message').html(translated_message);
			} else if (json.error.message == 'invalid city_key') {
				translated_message = 'Selecteer eerst een plaats';
				$(obj).find('.street_message').html(translated_message);
			}

		}
	}
})(jQuery);
